@@ -18,7 +18,7 @@ require Exporter;
 require DynaLoader;
 
 @ISA = qw(Exporter DynaLoader);
-$VERSION = '0.80';
+$VERSION = '0.90';
 
 bootstrap DBD::Firebird $VERSION;
 
@@ -98,6 +98,34 @@ sub _OdbcParse($$$)
     $hash->{database} = "$hash->{host}:$hash->{database}" if $hash->{host};
 }
 
+sub create_database {
+    my ( $self, $params ) = ( shift, shift );
+    $self and $params and ref($params) and ref($params) eq 'HASH' and not @_
+        or croak 'Usage: '
+        . __PACKAGE__
+        . '->create_database( { params...} )';
+
+    exists $params->{db_path} and defined( $params->{db_path} )
+        or croak "Required parameter 'db_path' not supplied";
+
+    for( qw(db_path user password character_set) ) {
+        next unless exists $params->{$_};
+
+        $params->{$_} =~ s/'/''/g if defined($params->{$_});
+    }
+
+    DBD::Firebird::db::_create_database($params);
+}
+
+sub gfix {
+    my ( $self, $params ) = ( shift, shift );
+    $self and $params and ref($params) and ref($params) eq 'HASH' and not @_
+        or croak 'Usage: '
+        . __PACKAGE__
+        . '->gfix( { params...} )';
+
+    DBD::Firebird::db::_gfix($params);
+}
 
 package DBD::Firebird::dr;
 
@@ -1124,6 +1152,15 @@ Retrieve query plan from a prepared SQL statement.
  my $sth = $dbh->prepare('SELECT * FROM foo');
  print $sth->func('ib_plan'); # PLAN (FOO NATURAL)
 
+=item C<ib_drop_database>
+
+ $result = $dbh->func('ib_drop_database');
+
+Drops the database, associated with the connection. The database handle is no
+longer valid after calling this function.
+
+Caution is advised as the drop is irrevocable.
+
 =back
 
 
@@ -1174,6 +1211,95 @@ C<Tie::DBI>. C<Tie::DBI> calls $dbh->prepare("LISTFIELDS $table_name") on
 which Firebird fails to parse. I think that the call should be made within 
 an eval block.
 
+=head1 SERVICE METHODS
+
+=head2 DBD::Firebird->create_database( { params... } )
+
+A class method for creating empty databases.
+
+The method croaks on error. Params may be:
+
+=over
+
+=item db_path (string, required)
+
+Path to database, including host name if necessary.
+
+Examples:
+
+=over
+
+=item server:/path/to/db.fdb
+
+=item /srv/db/base.fdb
+
+=back
+
+=item user (string, optional)
+
+User name to be used for the request.
+
+=item password (string, optional)
+
+Password to be used for the request.
+
+=item page_size (integer, optional)
+
+Page size of the newly created database. Should be something supported by the
+server. Firebird 2.5 supports the following page sizes: 1024, 2048, 4096, 8192
+and 16384 and defaults to 4096.
+
+=item character_set (string, optional)
+
+The default character set of the database. Firebird 2.5 defaults to C<NONE>.
+
+=item dialect (integer, optional)
+
+The dialect of the database. Defaults to 3.
+
+=back
+
+After creation, the new database can be used after connecting to it with the
+usual DBI->connect(...)
+
+=head2 DBD::Firebird->gfix( { params } )
+
+A class method for simulating a subset of the functionality of the
+Firebird's L<gfix(1)> utility.
+
+Params is a hash reference, with the following keys:
+
+=over
+
+=item db_path (string, required)
+
+The path to the database to connect to. Should include host name if
+necessary.
+
+=item user (string, optional)
+
+User name to connect as. Must be SYSDBA or database owner.
+
+=item password (string, optional)
+
+Password to be used for the connection.
+
+Note that user and password are not needed for embedded connections.
+
+=item forced_writes (boolean, optional)
+
+If given, sets the forced writes flag of the database, causing Firebird
+to use synchronous writes when working with that database.
+
+=item buffers (integer, optional)
+
+If given, sets the default number of buffers for the database. Can
+be overriden on connect time. Note that buffers are measured in database
+pages, not bytes.
+
+
+=back
+
 =head1 FAQ
 
 =head2 Why do some operations performing positioned update and delete fail when AutoCommit is on? 
@@ -1368,8 +1494,8 @@ C<set_tx_param()> is obsoleted by C<ib_set_tx_param()>.
 
 =item * DBI by Tim Bunce <Tim.Bunce@pobox.com>
 
-=item * DBD::Firebird by Edwin Pratomo <edpratomo@cpan.org> and Daniel Ritz 
-<daniel.ritz@gmx.ch>.
+=item * DBD::Firebird by Edwin Pratomo <edpratomo@cpan.org>, Daniel Ritz
+<daniel.ritz@gmx.ch> and many others. See L</COPYRIGHT & LICENSE>.
 
 This module is originally based on the work of Bill Karwin's IBPerl.
 
